'use strict';
import io from 'socket.io-client';

const SERVER_URL = 'http://' + process.env.VUE_APP_SERVER_HOST;
const SERVER_PORT = process.env.VUE_APP_SERVER_PORT;

export default function SocketService(socketEventBus, vuexStore) {
  let eventBus = socketEventBus;
  let store = vuexStore;
  let ioClient = null;
  let connect = function(name) {
    let promise = new Promise((resolve, reject) => {
      console.log(
        'Connecting socket service to : ' + SERVER_URL + ':' + SERVER_PORT
      );
      ioClient = io(SERVER_URL + ':' + SERVER_PORT, {
        reconnectionAttempts: 2
      });

      ioClient.on('disconnect', reason => {
        if (reason !== 'io client disconnect') {
          console.log('Connection with server lost unexpectidly : ' + reason);
          // this.ioClient.close();
        }
      });

      ioClient.on('connect', () => {
        console.log('Connected to server');
        ioClient.emit('auth', name, function(response) {
          if (response.res === 'ok') {
            resolve(response.message);
          } else {
            reject(response.message);
          }
        });
      });

      ioClient.on('connect_error', () => {
        console.log('connect_error to server');
      });

      ioClient.on('reconnect_failed', () => {
        console.log('reconnect_failed to server');
        reject(new Error('Cannot reach server'));
      });

      ioClient.on('reload-games-list', () => {
        console.log('force reload games list !');
        eventBus.$emit('reload-games');
      });
      ioClient.on('message', message => {
        store.dispatch('addMessageToQueue', message);
      });
    });
    return promise;
  };

  let checkConnection = function(playername) {
    if (ioClient && ioClient.connected === false) {
      return connect(playername);
    } else {
      return Promise.resolve('Still connected');
    }
  };

  let disconnect = function() {
    console.log('Manual client disconnect');
    ioClient.close();
  };
  let getGamesList = function(playername) {
    return checkConnection(playername).then(
      () => {
        return new Promise((resolve, reject) => {
          ioClient.emit('games-list', playername, function(response) {
            if (response.res === 'ok') {
              resolve(response.message);
            } else {
              reject(response.message);
            }
          });
        });
      },
      error => Promise.reject(error)
    );
  };

  let createGame = function(game) {
    return checkConnection(game.player1).then(
      () => {
        return new Promise((resolve, reject) => {
          ioClient.emit('create-game', game, function(response) {
            if (response.res === 'ok') {
              resolve(response.message);
            } else {
              reject(response.message);
            }
          });
        });
      },
      error => Promise.reject(error)
    );
  };
  let joinGame = function(username, gameId, joinCreatedGame) {
    console.log('gameId :>> ', gameId);
    return checkConnection(username).then(
      () => {
        return new Promise((resolve, reject) => {
          ioClient.emit('join-game', { id: gameId, joinCreatedGame }, function(
            response
          ) {
            if (response.res === 'ok') {
              resolve(response.message);
            } else {
              reject(response.message);
            }
          });
        });
      },
      error => Promise.reject(error)
    );
  };
  let leaveGame = function(username) {
    return checkConnection(username).then(
      () => {
        return new Promise(resolve => {
          ioClient.emit('leave-game', username, function(response) {
            resolve(response);
          });
        });
      },
      error => Promise.reject(error)
    );
  };

  let chat = function(message) {
    ioClient.emit('chat', message);
  };
  return {
    connect,
    disconnect,
    getGamesList,
    createGame,
    leaveGame,
    joinGame,
    chat
  };
}
