import { initHeroesFromJson } from '../../server-shared/heroesHelper';
import allHeroesJson from '../../server-shared/all-heroes.json';
const state = {
  heroes: new Set()
};

const getters = {
  all(state) {
    return state.heroes;
  },
  by: state => filter => {
    // filter has
    //  - faction (orcs humans elves meca none all)
    //  - popularity (with without)
    //  - draft (yes no all)
    //  -  minPower <= power <= maxPower
    //  -  minCost <= cost <= maxCost
    //  - byName : if true filter name with filter.name
    if (!filter.faction) filter.faction = 'all';
    if (!filter.popularity) filter.popularity = 'without';
    if (!filter.minPower) filter.minPower = 0;
    if (!filter.maxPower) filter.maxPower = 0;
    if (!filter.minCost) filter.minCost = 0;
    if (!filter.maxCost) filter.minCost = 0;

    let draftFilter = 'all';
    switch (filter.draft) {
      case 'yes':
        draftFilter = true;
        break;
      case 'no':
        draftFilter = false;
        break;

      default:
        draftFilter = 'all';
        break;
    }

    return Array.from(state.heroes).filter(hero => {
      if (filter.faction !== 'all' && hero.faction !== filter.faction)
        return false;
      if (hero.popularity !== 'any' && hero.popularity !== filter.popularity)
        return false;

      if (draftFilter !== 'all' && hero.isDraftable !== draftFilter)
        return false;

      if (hero.power < filter.minPower || hero.power > filter.maxPower)
        return false;
      if (hero.cost < filter.minCost || hero.cost > filter.maxCost)
        return false;

      if (
        filter.byName === true &&
        !hero.name.toLowerCase().includes(filter.name.toLowerCase())
      )
        return false;

      return true;
    });
  }
};

const mutations = {
  SET_HEROES: (state, payload) => {
    state.heroes = payload;
  }
};

const actions = {
  setFromLocalJson: ({ commit }) => {
    commit('SET_HEROES', initHeroesFromJson(allHeroesJson));
  },
  setAvailableHeroes: ({ commit }, payload) => {
    commit('SET_HEROES', payload);
  }
};
export default {
  namespaced: true,
  state,
  getters,
  mutations,
  actions
};
